package eu.lucazanini;

import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import android.app.ActionBar;
import android.app.ActionBar.Tab;
import android.app.Activity;
import android.app.Fragment;
import android.app.FragmentManager;
import android.app.FragmentTransaction;
import android.os.Bundle;
import android.view.View;

public class TabActionBarActivity extends Activity {

    private class TabListener<T extends Fragment> implements
	    ActionBar.TabListener {
	private final Activity mActivity;
	private final Class<T> mClass;
	private Fragment mFragment;
	private final String mTag;

	/**
	 * Constructor used each time a new tab is created.
	 * 
	 * @param activity
	 *            The host Activity, used to instantiate the fragment
	 * @param tag
	 *            The identifier tag for the fragment
	 * @param clz
	 *            The fragment's Class, used to instantiate the fragment
	 */
	public TabListener(Activity activity, String tag, Class<T> clz) {
	    mActivity = activity;
	    mTag = tag;
	    mClass = clz;
	}

	@Override
	public void onTabReselected(Tab tab, FragmentTransaction ft) {
	    // User selected the already selected tab. Usually do nothing.
	}

	@Override
	public void onTabSelected(Tab tab, FragmentTransaction ft) {
	    // Check if the fragment is already initialized
	    Iterator<WeakReference<Fragment>> it = fragList.iterator();
	    while (it.hasNext()) {
		Fragment f = it.next().get();
		if (f.getTag().equals(mTag)) {

		    mFragment = f;
		} else {
		    ft.detach(f);
		}
	    }

	    if (mFragment == null) {
		// If not, instantiate and add it to the activity
		mFragment = Fragment.instantiate(mActivity, mClass.getName());
		ft.add(android.R.id.content, mFragment, mTag);
	    } else {
		// If it exists, simply attach it in order to show it
		ft.attach(mFragment);
	    }
	}

	@Override
	public void onTabUnselected(Tab tab, FragmentTransaction ft) {
	    if (mFragment != null) {
		// Detach the fragment, because another one is being attached
		ft.detach(mFragment);
	    }
	}

    }

    List<WeakReference<Fragment>> fragList = new ArrayList<WeakReference<Fragment>>();

    @Override
    public void onAttachFragment(Fragment fragment) {
	fragList.add(new WeakReference<Fragment>(fragment));
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
	super.onCreate(savedInstanceState);

	ActionBar actionBar = getActionBar();

	actionBar.setNavigationMode(ActionBar.NAVIGATION_MODE_TABS);

	String label1 = getResources().getString(R.string.label1);
	Tab tab = actionBar.newTab();
	tab.setText(label1);

	TabListener<Tab1Fragment> tl = new TabListener<Tab1Fragment>(this,
		label1, Tab1Fragment.class);
	tab.setTabListener(tl);
	actionBar.addTab(tab);

	String label2 = getResources().getString(R.string.label2);
	tab = actionBar.newTab();
	tab.setText(label2);

	TabListener<Tab2Fragment> tl2 = new TabListener<Tab2Fragment>(this,
		label2, Tab2Fragment.class);
	tab.setTabListener(tl2);
	actionBar.addTab(tab);

    }

    public void replaceFragment(View v) {

	String label1;
	FragmentManager fm;
	FragmentTransaction ft;
	Iterator<WeakReference<Fragment>> it;
	int id = v.getId();

	switch (id) {
	case R.id.btn1:
	    label1 = getResources().getString(R.string.label1);
	    Fragment f3 = new Tab3Fragment();

	    fm = getFragmentManager();
	    ft = fm.beginTransaction();
	    ft.replace(android.R.id.content, f3, label1);
	    ft.commit();

	    it = fragList.iterator();
	    while (it.hasNext()) {
		WeakReference<Fragment> ref = it.next();
		Fragment f = ref.get();
		if (f instanceof Tab1Fragment) {
		    it.remove();
		}
	    }

	    break;
	case R.id.btn3:
	    label1 = getResources().getString(R.string.label1);
	    Fragment f1 = new Tab1Fragment();

	    fm = getFragmentManager();
	    ft = fm.beginTransaction();
	    ft.replace(android.R.id.content, f1, label1);
	    ft.commit();

	    it = fragList.iterator();
	    while (it.hasNext()) {
		WeakReference<Fragment> ref = it.next();
		Fragment f = ref.get();
		if (f instanceof Tab3Fragment) {
		    it.remove();
		}
	    }

	    break;
	default:
	    break;
	}

    }
}